@@ -1,7 +1,7 @@
 package Dist::Zilla::Plugin::Rinci::AbstractFromMeta;
 
-our $DATE = '2014-12-14'; # DATE
-our $VERSION = '0.02'; # VERSION
+our $DATE = '2014-12-16'; # DATE
+our $VERSION = '0.03'; # VERSION
 
 use 5.010001;
 use strict;
@@ -19,46 +19,22 @@ with (
 );
 
 use Data::Dump qw(dump);
+use File::Slurp::Tiny qw(read_file);
 use File::Spec::Functions qw(catfile);
 
-sub _get_abstract_from_meta {
+sub _get_from_module {
     my ($self, $filename) = @_;
 
-    local @INC = @INC;
-    unshift @INC, 'lib';
-
-    my $content = do {
-        open my($fh), "<", $filename or die "Can't open $filename: $!";
-        local $/;
-        ~~<$fh>;
-    };
-
-    unless ($content =~ m{^#[ \t]*ABSTRACT:[ \t]*([^\n]*)[ \t]*$}m) {
-        return undef;
-    }
-
-    my $abstract = $1;
-    if ($abstract =~ /\S/) {
-        #$self->log_debug(["skipping %s: Abstract already filled (%s)", $filename, $abstract]);
-        return $abstract;
-    }
-
-    # find out the package of the file
-    my $pkg;
-    if ($content =~ m{^\s*package\s+(\w+(?:::\w+)*)\s*;}m) {
-        $pkg = $1;
-    } else {
-        $pkg = 'main';
-    }
-
-    # XXX if script, do()
     (my $mod_p = $filename) =~ s!^lib/!!;
     require $mod_p;
 
+    # find out the package of the file
+    (my $pkg = $mod_p) =~ s/\.pm\z//; $pkg =~ s!/!::!g;
+
     no strict 'refs';
     my $metas = \%{"$pkg\::SPEC"};
 
-    $abstract = undef;
+    my $abstract;
     {
         if ($metas->{':package'}) {
             $abstract = $metas->{':package'}{summary};
@@ -76,13 +52,103 @@ sub _get_abstract_from_meta {
             last if $abstract;
         }
     }
+    return $abstract;
+}
 
-    #$self->log_debug(["Figured out abstract for %s: %s", $filename, $abstract])
-    #    if $abstract;
+sub _get_from_script {
+    my ($self, $filename) = @_;
+
+    # check if script uses Perinci::CmdLine
+    my $ct = read_file $filename;
+    unless ($ct =~ /\b(?:use|require)\s+
+                    (Perinci::CmdLine(?:::Lite|::Any)?)\b/x) {
+        return undef;
+    }
+
+    require UUID::Random;
+    my $tag=UUID::Random::generate();
+
+    my @cmd = ($^X, "-Ilib", "-MPerinci::CmdLine::Base::Patch::DumpAndExit=-tag,$tag");
+    push @cmd, $filename;
+    push @cmd, "--version";
+    require Capture::Tiny;
+    my ($stdout, $stderr, $exit) = Capture::Tiny::capture(
+        sub { system @cmd },
+    );
+    my $cli;
+    if ($stdout =~ /^# BEGIN DUMP $tag\s+(.*)^# END DUMP $tag/ms) {
+        $cli = eval $1;
+        if ($@) {
+            die "Script '$filename' detected as using Perinci::CmdLine, ".
+                "but error in eval-ing captured object: $@, ".
+                    "raw captured object: <<<$1>>>";
+        }
+        if (!blessed($cli)) {
+            die "Script '$filename' detected as using Perinci::CmdLine, ".
+                "but didn't get an object?, raw captured output=<<$stdout>>";
+        }
+    } else {
+        die "Script '$filename' detected as using Perinci::CmdLine, ".
+            "but can't capture object, raw captured output: stdout=<<$stdout>>, stderr=<<$stderr>>";
+    }
+
+    return $cli->{summary} if $cli->{summary};
+    return undef unless $cli->{url};
+
+    # XXX handle embedded but not in /main
+    if ($cli->{url} =~ m!^(pl:)?/main/!) {
+        # function is embedded in script (/main/FOO), we need to load the
+        # metadata in-process
+        no warnings;
+        %main::SPEC = (); # empty first to avoid mixing with other scripts'
+        (undef, undef, undef) = Capture::Tiny::capture(sub {
+            eval q{package main; use Perinci::CmdLine::Base::Patch::DumpAndExit -tag=>'$tag', -exit_method=>'die'; do "$filename"};
+        });
+    }
+    state $pa = do { require Perinci::Access; Perinci::Access->new };
+    my $res = $pa->request(meta => $cli->{url});
+    die "Can't meta $cli->{url}: $res->[0] - $res->[1]" unless $res->[0] == 200;
+    my $meta = $res->[2];
+
+    return $meta->{summary};
+}
+
+sub _get_abstract_from_meta {
+    my ($self, $filename) = @_;
+
+    local @INC = @INC;
+    unshift @INC, 'lib';
+
+    my $content = do {
+        open my($fh), "<", $filename or die "Can't open $filename: $!";
+        local $/;
+        ~~<$fh>;
+    };
+
+    unless ($content =~ m{^#[ \t]*ABSTRACT:[ \t]*([^\n]*)[ \t]*$}m) {
+        $self->log_debug(["Skipping %s: no # ABSTRACT", $filename]);
+        return undef;
+    }
+
+    my $abstract = $1;
+    if ($abstract =~ /\S/) {
+        $self->log_debug(["Skipping %s: Abstract already filled (%s)", $filename, $abstract]);
+        return $abstract;
+    }
+
+    # XXX if script, do()
+    if ($filename =~ m!^lib/!) {
+        $self->log_debug(["Getting abstract for module %s", $filename]);
+        $abstract = $self->_get_from_module($filename);
+    } else {
+        $self->log_debug(["Getting abstract for script %s", $filename]);
+        $abstract = $self->_get_from_script($filename);
+    }
     $abstract;
 }
 
-# btw, why does dzil need to know abstract for main module before build?
+# dzil also wants to get abstract for main module to put in dist's
+# META.{yml,json}
 sub before_build {
    my $self  = shift;
    my $name  = $self->zilla->name;
@@ -110,13 +176,6 @@ sub munge_file {
     my ($self, $file) = @_;
     my $content = $file->content;
 
-    # we don't support execfiles for now because we need to patch pericmd* first
-    # to dump instead of run
-    unless ($file->name =~ m!^lib/!) {
-        $self->log_debug(["skipping %s for now: not a module", $file->name]);
-        return;
-    }
-
     my $abstract = $self->_get_abstract_from_meta($file->name);
     unless (defined $abstract) {
         die "Can't figure out abstract for " . $file->name;
@@ -144,7 +203,7 @@ Dist::Zilla::Plugin::Rinci::AbstractFromMeta - Fill out abstract from Rinci meta
 
 =head1 VERSION
 
-This document describes version 0.02 of Dist::Zilla::Plugin::Rinci::AbstractFromMeta (from Perl distribution Dist-Zilla-Plugin-Rinci-AbstractFromMeta), released on 2014-12-14.
+This document describes version 0.03 of Dist::Zilla::Plugin::Rinci::AbstractFromMeta (from Perl distribution Dist-Zilla-Plugin-Rinci-AbstractFromMeta), released on 2014-12-16.
 
 =head1 SYNOPSIS
 
